@@ -54,6 +54,7 @@ from pymtl3.passes.testcases import (
     CaseTwoUpblksFreevarsComp,
     CaseTwoUpblksSliceComp,
     CaseTwoUpblksStructTmpWireComp,
+    CaseUpdateffMixAssignComp,
     CaseWiresDrivenComp,
     set_attributes,
 )
@@ -255,7 +256,7 @@ CaseBits32FreeVarToTmpVarComp = set_attributes( CaseBits32FreeVarToTmpVarComp,
     'REF_TMPVAR',
     '''\
         tmpvars:
-          tmpvar: u in upblk of Vector32
+          tmpvar: u in upblk of Vector1
     ''',
     'REF_SRC',
     '''\
@@ -271,7 +272,7 @@ CaseBits32FreeVarToTmpVarComp = set_attributes( CaseBits32FreeVarToTmpVarComp,
         wire_decls:
         component_decls:
         tmpvars:
-          tmpvar: u in upblk of Vector32
+          tmpvar: u in upblk of Vector1
         upblk_srcs:
           upblk_src: upblk
         connections:
@@ -326,7 +327,7 @@ CaseBits32ConstIntToTmpVarComp = set_attributes( CaseBits32ConstIntToTmpVarComp,
     'REF_TMPVAR',
     '''\
         tmpvars:
-          tmpvar: u in upblk of Vector32
+          tmpvar: u in upblk of Vector1
     ''',
     'REF_SRC',
     '''\
@@ -341,7 +342,7 @@ CaseBits32ConstIntToTmpVarComp = set_attributes( CaseBits32ConstIntToTmpVarComp,
         wire_decls:
         component_decls:
         tmpvars:
-          tmpvar: u in upblk of Vector32
+          tmpvar: u in upblk of Vector1
         upblk_srcs:
           upblk_src: upblk
         connections:
@@ -361,15 +362,15 @@ CaseStructTmpWireComp = set_attributes( CaseStructTmpWireComp,
     'REF_TMPVAR',
     '''\
         tmpvars:
-          tmpvar: u in upblk of Struct Bits32Foo
+          tmpvar: u in upblk of Struct Bits32Foo__foo_32
     ''',
     'REF_SRC',
     '''\
-        struct Bits32Foo
+        struct Bits32Foo__foo_32
         component DUT
         (
         port_decls:
-          port_decl: in_ Port of Struct Bits32Foo
+          port_decl: in_ Port of Struct Bits32Foo__foo_32
           port_decl: out Port of Vector32
         interface_decls:
         );
@@ -378,7 +379,7 @@ CaseStructTmpWireComp = set_attributes( CaseStructTmpWireComp,
         wire_decls:
         component_decls:
         tmpvars:
-          tmpvar: u in upblk of Bits32Foo
+          tmpvar: u in upblk of Bits32Foo__foo_32
         upblk_srcs:
           upblk_src: upblk
         connections:
@@ -399,18 +400,18 @@ CaseTwoUpblksStructTmpWireComp = set_attributes( CaseTwoUpblksStructTmpWireComp,
     'REF_TMPVAR',
     '''\
         tmpvars:
-          tmpvar: u in multi_upblks_1 of Struct Bits32Foo
-          tmpvar: u in multi_upblks_2 of Struct Bits32Bar
+          tmpvar: u in multi_upblks_1 of Struct Bits32Foo__foo_32
+          tmpvar: u in multi_upblks_2 of Struct Bits32Bar__bar_32
     ''',
     'REF_SRC',
     '''\
-        struct Bits32Foo
-        struct Bits32Bar
+        struct Bits32Foo__foo_32
+        struct Bits32Bar__bar_32
         component DUT
         (
         port_decls:
-          port_decl: in_bar Port of Struct Bits32Bar
-          port_decl: in_foo Port of Struct Bits32Foo
+          port_decl: in_bar Port of Struct Bits32Bar__bar_32
+          port_decl: in_foo Port of Struct Bits32Foo__foo_32
           port_decl: out_bar Port of Vector32
           port_decl: out_foo Port of Vector32
         interface_decls:
@@ -420,8 +421,8 @@ CaseTwoUpblksStructTmpWireComp = set_attributes( CaseTwoUpblksStructTmpWireComp,
         wire_decls:
         component_decls:
         tmpvars:
-          tmpvar: u in multi_upblks_1 of Bits32Foo
-          tmpvar: u in multi_upblks_2 of Bits32Bar
+          tmpvar: u in multi_upblks_1 of Bits32Foo__foo_32
+          tmpvar: u in multi_upblks_2 of Bits32Bar__bar_32
         upblk_srcs:
           upblk_src: multi_upblks_1
           upblk_src: multi_upblks_2
@@ -480,11 +481,11 @@ CaseStructIfcTmpVarOutComp = set_attributes( CaseStructIfcTmpVarOutComp,
     'REF_TMPVAR',
     '''\
         tmpvars:
-          tmpvar: u in upblk of Struct Bits32Foo
+          tmpvar: u in upblk of Struct Bits32Foo__foo_32
     ''',
     'REF_SRC',
     '''\
-        struct Bits32Foo
+        struct Bits32Foo__foo_32
         component DUT
         (
         port_decls:
@@ -492,14 +493,14 @@ CaseStructIfcTmpVarOutComp = set_attributes( CaseStructIfcTmpVarOutComp,
         interface_decls:
           interface_decl: ifc InterfaceView Bits32FooInIfc
             interface_ports:
-              interface_port: foo Port of Struct Bits32Foo
+              interface_port: foo Port of Struct Bits32Foo__foo_32
         );
         const_decls:
         freevars:
         wire_decls:
         component_decls:
         tmpvars:
-          tmpvar: u in upblk of Bits32Foo
+          tmpvar: u in upblk of Bits32Foo__foo_32
         upblk_srcs:
           upblk_src: upblk
         connections:
@@ -1128,7 +1129,7 @@ CaseStructPortOnly = set_attributes( CaseStructPortOnly,
     'REF_PORT',
     '''\
         port_decls:
-          port_decl: in_ Port of Struct Bits32Foo
+          port_decl: in_ Port of Struct Bits32Foo__foo_32
     ''',
     'REF_WIRE',
     'wire_decls:\n',
@@ -1138,11 +1139,11 @@ CaseStructPortOnly = set_attributes( CaseStructPortOnly,
     'connections:\n',
     'REF_SRC',
     '''\
-        struct Bits32Foo
+        struct Bits32Foo__foo_32
         component DUT
         (
         port_decls:
-          port_decl: in_ Port of Struct Bits32Foo
+          port_decl: in_ Port of Struct Bits32Foo__foo_32
         interface_decls:
         );
         const_decls:
@@ -1156,7 +1157,7 @@ CaseStructPortOnly = set_attributes( CaseStructPortOnly,
         endcomponent
     ''',
     'REF_STRUCT',
-    [(rdt.Struct('Bits32Foo', {'foo':rdt.Vector(32)}), 'Bits32Foo')]
+    [(rdt.Struct('Bits32Foo', {'foo':rdt.Vector(32)}), 'Bits32Foo__foo_32')]
 )
 
 CaseStructWireDrivenComp = set_attributes( CaseStructWireDrivenComp,
@@ -1167,7 +1168,7 @@ CaseStructWireDrivenComp = set_attributes( CaseStructWireDrivenComp,
     'REF_WIRE',
     '''\
         wire_decls:
-          wire_decl: foo Wire of Struct Bits32Foo
+          wire_decl: foo Wire of Struct Bits32Foo__foo_32
     ''',
     'REF_CONST',
     'const_decls:\n',
@@ -1175,7 +1176,7 @@ CaseStructWireDrivenComp = set_attributes( CaseStructWireDrivenComp,
     'connections:\n',
     'REF_SRC',
     '''\
-        struct Bits32Foo
+        struct Bits32Foo__foo_32
         component DUT
         (
         port_decls:
@@ -1184,7 +1185,7 @@ CaseStructWireDrivenComp = set_attributes( CaseStructWireDrivenComp,
         const_decls:
         freevars:
         wire_decls:
-          wire_decl: foo Wire of Struct Bits32Foo
+          wire_decl: foo Wire of Struct Bits32Foo__foo_32
         component_decls:
         tmpvars:
         upblk_srcs:
@@ -1194,7 +1195,7 @@ CaseStructWireDrivenComp = set_attributes( CaseStructWireDrivenComp,
         endcomponent
     ''',
     'REF_STRUCT',
-    [(rdt.Struct('Bits32Foo', {'foo':rdt.Vector(32)}), 'Bits32Foo')]
+    [(rdt.Struct('Bits32Foo', {'foo':rdt.Vector(32)}), 'Bits32Foo__foo_32')]
 )
 
 CaseStructConstComp = set_attributes( CaseStructConstComp,
@@ -1207,7 +1208,7 @@ CaseStructConstComp = set_attributes( CaseStructConstComp,
     'REF_CONST',
     '''\
         const_decls:
-          const_decl: struct_const Const of Struct Bits32Foo
+          const_decl: struct_const Const of Struct Bits32Foo__foo_32
     ''',
     'REF_CONN',
     'connections:\n',
@@ -1229,7 +1230,7 @@ CaseStructConstComp = set_attributes( CaseStructConstComp,
         endcomponent
     ''',
     'REF_STRUCT',
-    [(rdt.Struct('Bits32Foo', {'foo':rdt.Vector(32)}), 'Bits32Foo')]
+    [(rdt.Struct('Bits32Foo', {'foo':rdt.Vector(32)}), 'Bits32Foo__foo_32')]
 )
 
 CaseStructx5PortOnly = set_attributes( CaseStructx5PortOnly,
@@ -1248,7 +1249,7 @@ CaseStructx5PortOnly = set_attributes( CaseStructx5PortOnly,
     'connections:\n',
     'REF_SRC',
     '''\
-        struct Bits32Foo
+        struct Bits32Foo__foo_32
         component DUT
         (
         port_decls:
@@ -1266,7 +1267,7 @@ CaseStructx5PortOnly = set_attributes( CaseStructx5PortOnly,
         endcomponent
     ''',
     'REF_STRUCT',
-    [(rdt.Struct('Bits32Foo', {'foo':rdt.Vector(32)}), 'Bits32Foo')]
+    [(rdt.Struct('Bits32Foo', {'foo':rdt.Vector(32)}), 'Bits32Foo__foo_32')]
 )
 
 CaseNestedStructPortOnly = set_attributes( CaseNestedStructPortOnly,
@@ -1275,7 +1276,7 @@ CaseNestedStructPortOnly = set_attributes( CaseNestedStructPortOnly,
     'REF_PORT',
     '''\
         port_decls:
-          port_decl: in_ Port of Struct NestedBits32Foo
+          port_decl: in_ Port of Struct NestedBits32Foo__foo_Bits32Foo__foo_32
     ''',
     'REF_WIRE',
     'wire_decls:\n',
@@ -1285,12 +1286,12 @@ CaseNestedStructPortOnly = set_attributes( CaseNestedStructPortOnly,
     'connections:\n',
     'REF_SRC',
     '''\
-        struct Bits32Foo
-        struct NestedBits32Foo
+        struct Bits32Foo__foo_32
+        struct NestedBits32Foo__foo_Bits32Foo__foo_32
         component DUT
         (
         port_decls:
-          port_decl: in_ Port of Struct NestedBits32Foo
+          port_decl: in_ Port of Struct NestedBits32Foo__foo_Bits32Foo__foo_32
         interface_decls:
         );
         const_decls:
@@ -1305,8 +1306,8 @@ CaseNestedStructPortOnly = set_attributes( CaseNestedStructPortOnly,
     ''',
     'REF_STRUCT',
     [
-      (rdt.Struct('Bits32Foo', {'foo':rdt.Vector(32)}), 'Bits32Foo'),
-      (rdt.Struct('NestedBits32Foo', {'foo':rdt.Struct('Bits32Foo', {'foo':rdt.Vector(32)})}), 'NestedBits32Foo'),
+      (rdt.Struct('Bits32Foo', {'foo':rdt.Vector(32)}), 'Bits32Foo__foo_32'),
+      (rdt.Struct('NestedBits32Foo', {'foo':rdt.Struct('Bits32Foo', {'foo':rdt.Vector(32)})}), 'NestedBits32Foo__foo_Bits32Foo__foo_32'),
     ]
 )
 
@@ -1316,8 +1317,8 @@ CaseNestedPackedArrayStructComp = set_attributes( CaseNestedPackedArrayStructCom
     'REF_PORT',
     '''\
         port_decls:
-          port_decl: in_ Port of Struct NestedStructPackedArray
-          port_decl: out Port of Struct Bits32x5Foo
+          port_decl: in_ Port of Struct NestedStructPackedArray__foo_Bits32x5Foo__foo_32x5x5
+          port_decl: out Port of Struct Bits32x5Foo__foo_32x5
     ''',
     'REF_WIRE',
     'wire_decls:\n',
@@ -1330,13 +1331,13 @@ CaseNestedPackedArrayStructComp = set_attributes( CaseNestedPackedArrayStructCom
     ''',
     'REF_SRC',
     '''\
-        struct Bits32x5Foo
-        struct NestedStructPackedArray
+        struct Bits32x5Foo__foo_32x5
+        struct NestedStructPackedArray__foo_Bits32x5Foo__foo_32x5x5
         component DUT
         (
         port_decls:
-          port_decl: in_ Port of Struct NestedStructPackedArray
-          port_decl: out Port of Struct Bits32x5Foo
+          port_decl: in_ Port of Struct NestedStructPackedArray__foo_Bits32x5Foo__foo_32x5x5
+          port_decl: out Port of Struct Bits32x5Foo__foo_32x5
         interface_decls:
         );
         const_decls:
@@ -1352,8 +1353,8 @@ CaseNestedPackedArrayStructComp = set_attributes( CaseNestedPackedArrayStructCom
     ''',
     'REF_STRUCT',
     [
-      (rdt.Struct('Bits32x5Foo', {'foo':rdt.PackedArray([5], rdt.Vector(32))}), 'Bits32x5Foo'),
-      (rdt.Struct('NestedStructPackedArray', {'foo':rdt.Struct('Bits32x5Foo', {'foo':rdt.PackedArray([5], rdt.Vector(32))})}), 'NestedStructPackedArray'),
+      (rdt.Struct('Bits32x5Foo', {'foo':rdt.PackedArray([5], rdt.Vector(32))}), 'Bits32x5Foo__foo_32x5'),
+      (rdt.Struct('NestedStructPackedArray', {'foo':rdt.PackedArray([5], rdt.Struct('Bits32x5Foo', {'foo':rdt.PackedArray([5], rdt.Vector(32))}))}), 'NestedStructPackedArray__foo_Bits32x5Foo__foo_32x5x5'),
     ]
 )
 
