@@ -11,6 +11,8 @@ from copy import deepcopy
 from pymtl3.passes.PassConfigs import Checker
 from pymtl3.passes.PlaceholderConfigs import PlaceholderConfigs, expand
 
+from .VerilogPlaceholderPass import VerilogPlaceholderPass
+
 
 class VerilogPlaceholderConfigs( PlaceholderConfigs ):
 
@@ -42,6 +44,9 @@ class VerilogPlaceholderConfigs( PlaceholderConfigs ):
     # -I ( alias of -y and +incdir+ )
     # Expects a list of include paths; [] to disable this option
     "v_include" : [],
+
+    # The separator used for name mangling
+    "separator" : '__',
   }
 
   VerilogCheckers = {
@@ -62,12 +67,12 @@ class VerilogPlaceholderConfigs( PlaceholderConfigs ):
                             "expects a list of paths to directory"),
   }
 
-  PassName = 'VerilogPlaceholderConfigs'
+  Pass = VerilogPlaceholderPass
 
-  def __new__( cls, *args, **kwargs ):
+  def __new__( cls, m ):
     inst = super().__new__( cls )
-    assert len(args) == 0, "We only accept keyword arguments here."
 
+    # Do not pollute the attributes of the parent class
     cls.Options  = deepcopy( PlaceholderConfigs.Options )
     cls.Checkers = deepcopy( PlaceholderConfigs.Checkers )
 
@@ -76,8 +81,6 @@ class VerilogPlaceholderConfigs( PlaceholderConfigs ):
         f'config {key} is duplicated between PlaceholderConfigs and VerilogPlaceholderConfigs'
       cls.Options[key] = val
 
-    all_checkers = inst._get_all_checker_configs( cls )
-
     for cfgs, chk in cls.VerilogCheckers.items():
       if isinstance( cfgs, tuple ):
         for cfg in cfgs:
@@ -94,23 +97,13 @@ class VerilogPlaceholderConfigs( PlaceholderConfigs ):
     # Exactly one of src_file and v_flist should be non-empty
     if (s.v_flist and os.path.isfile(expand(s.src_file))) or \
        (not s.v_flist and not os.path.isfile(expand(s.src_file))):
-      raise InvalidPassOptionValue( 'src_file', s.src_file, s.PassName,
+      raise InvalidPassOptionValue( 'src_file', s.src_file, s.Pass.__name__,
           'exactly one of src_file and v_flist should be non-emtpy!' )
 
   def get_port_map( s ):
-    pmap = s.port_map
+    pmap = { p._dsl._my_name: name for p, name in s.port_map.items() }
     return lambda name: pmap[name] if name in pmap else name
 
-  def _get_all_checker_configs( s, cls ):
-    ret = []
-    for cfgs in cls.Checkers.keys():
-      if isinstance( cfgs, tuple ):
-        for cfg in cfgs:
-          ret.append( cfg )
-      elif isinstance( cfgs, str ):
-        ret.append( cfgs )
-    return ret
-
   def _add_to_checkers( s, checkers, cfg, chk ):
     assert cfg not in checkers,\
       f'config {cfg} is duplicated between PlaceholderConfigs!'
