@@ -14,10 +14,10 @@ from pymtl3.datatypes import Bits1, mk_bits
 from pymtl3.dsl import OutPort
 from pymtl3.passes.rtlir import RTLIRDataType as rdt
 from pymtl3.passes.rtlir import RTLIRType as rt
-from pymtl3.stdlib.test import TestVectorSimulator
+from pymtl3.stdlib.test_utils import TestVectorSimulator
 
-from ...yosys import TranslationImportPass as YosysTransImportPass
-from .. import TranslationImportPass as VTransImportPass
+from ...yosys import YosysTranslationImportPass
+from .. import VerilogTranslationImportPass
 
 #=========================================================================
 # test utility functions
@@ -290,7 +290,7 @@ def closed_loop_component_input_test( dut, test_vector, tv_in, backend = "verilo
   def ref_tv_out( model, test_vector ):
     dct = {}
     for out_port in all_output_ports:
-      dct[ out_port ] = eval( "model." + out_port._dsl.my_name )
+      dct[ out_port ] = eval( "model." + out_port._dsl.my_name ).clone() # WE NEED TO CLONE NOW
     reference_output.append( dct )
 
   # Method to compare the outputs of the imported model and the pure python one
@@ -312,11 +312,11 @@ def closed_loop_component_input_test( dut, test_vector, tv_in, backend = "verilo
     # If it simulates correctly, translate it and import it back
     dut.elaborate()
     if backend == "verilog":
-      dut.verilog_translate_import = True
-      imported_obj = VTransImportPass()( dut )
+      dut.set_metadata( VerilogTranslationImportPass.enable, True )
+      imported_obj = VerilogTranslationImportPass()( dut )
     elif backend == "yosys":
-      dut.yosys_translate_import = True
-      imported_obj = YosysTransImportPass()( dut )
+      dut.set_metadata( YosysTranslationImportPass.enable, True )
+      imported_obj = YosysTranslationImportPass()( dut )
     # Run another vector simulator spin
     imported_sim = TestVectorSimulator( imported_obj, test_vector, tv_in, tv_out )
     imported_sim.run_test()
@@ -346,7 +346,7 @@ def closed_loop_component_test( dut, data, backend = "verilog" ):
       # `setattr` fails to set the correct value of an array if indexed by
       # a subscript. We use `exec` here to make sure the value of elements
       # are assigned correctly.
-      exec( "model." + name + " = data" )
+      exec( "model." + name + " @= data" )
   test_vector = data.draw( DataStrategy( dut ) )
   closed_loop_component_input_test( dut, test_vector, tv_in, backend )
 
