@@ -25,8 +25,9 @@ def make_indent( src, nindent ):
 
 def get_component_unique_name( c_rtype ):
   full_name = get_component_full_name( c_rtype )
+  special_chars = [' ', '<', '>', '.']
 
-  if len( full_name ) < 64:
+  if len( full_name ) < 64 and not any([c in full_name for c in special_chars]):
     return full_name
 
   comp_name = c_rtype.get_name()
@@ -126,7 +127,7 @@ def get_rtype( _rtype ):
 # gen_mapped_ports
 #-----------------------------------------------------------------------
 
-def gen_mapped_ports( m, port_map, has_clk=True, has_reset=True ):
+def gen_mapped_ports( m, port_map, has_clk=True, has_reset=True, sep='__' ):
   """Return a list of (pname, vname, rt.Port/rt.Array ) that has all ports
   of `rtype`. This method performs SystemVerilog backend-specific name
   mangling and returns all ports that appear in the interface of component
@@ -139,6 +140,8 @@ def gen_mapped_ports( m, port_map, has_clk=True, has_reset=True ):
      element is mapped in port_map, or _all_ of the elements are mapped.
   """
 
+  port_map = {p._dsl._my_name:n for p, n in port_map.items()}
+
   # [pnames], vname, rtype, port_idx
 
   def _mangle_port( pname, vname, port, n_dim ):
@@ -161,7 +164,7 @@ def gen_mapped_ports( m, port_map, has_clk=True, has_reset=True ):
         all_ports.append( ( [_pname], _vname, _port, 0 ) )
       else:
         for i in range( _n_dim[0] ):
-          Q.append( (f"{_pname}[{i}]", f"{_vname}__{i}", _port, _n_dim[1:]) )
+          Q.append( (f"{_pname}[{i}]", f"{_vname}{sep}{i}", _port, _n_dim[1:]) )
 
     assert found == len(all_ports) or found == 0, \
         f"{pname} is an {len(n_dim)}-D array of ports with {len(all_ports)} ports in total, " \
@@ -181,7 +184,7 @@ def gen_mapped_ports( m, port_map, has_clk=True, has_reset=True ):
       _pname, _vname, _rtype, _n_dim, _prev_dims = Q.popleft()
       if _n_dim:
         for i in range(_n_dim[0]):
-          Q.append((f"{_pname}[{i}]", f"{_vname}__{i}", _rtype, _n_dim[1:], _prev_dims+1))
+          Q.append((f"{_pname}[{i}]", f"{_vname}{sep}{i}", _rtype, _n_dim[1:], _prev_dims+1))
       else:
         if isinstance( _rtype, rt.Port ):
           # Port inside the interface
@@ -195,7 +198,7 @@ def gen_mapped_ports( m, port_map, has_clk=True, has_reset=True ):
           # Interface (nested)
           for sub_name, sub_rtype in _rtype.get_all_properties_packed():
             sub_n_dim, sub_rtype = get_rtype( sub_rtype )
-            Q.append((f"{_pname}.{sub_name}", f"{_vname}__{sub_name}", sub_rtype, sub_n_dim, _prev_dims))
+            Q.append((f"{_pname}.{sub_name}", f"{_vname}{sep}{sub_name}", sub_rtype, sub_n_dim, _prev_dims))
         else:
           assert False, f"{_pname} is not interface(s) or port(s)!"
 
@@ -226,7 +229,7 @@ def gen_mapped_ports( m, port_map, has_clk=True, has_reset=True ):
           new_prev_n_dim = _prev_n_dim
           for sub_name, sub_rtype in _rtype.get_all_properties_packed():
             sub_n_dim, sub_rtype = get_rtype( sub_rtype )
-            Q.append((f"{_pname}.{sub_name}", f"{_vname}__{sub_name}",
+            Q.append((f"{_pname}.{sub_name}", f"{_vname}{sep}{sub_name}",
                       sub_rtype, sub_n_dim, new_prev_n_dim, _port_idx))
       else:
         assert False, f"{_pname} is not interface(s) or port(s)!"
