@@ -6,37 +6,67 @@
 """Provide L1 behavioral RTLIR type check pass."""
 
 import copy
+import math
+from collections import deque
+from contextlib import contextmanager
 
+from pymtl3 import MetadataKey
 from pymtl3.datatypes import Bits32, mk_bits
-from pymtl3.passes.BasePass import BasePass, PassMetadata
 from pymtl3.passes.rtlir.errors import PyMTLTypeError, RTLIRConversionError
+from pymtl3.passes.rtlir.RTLIRPass import RTLIRPass
 from pymtl3.passes.rtlir.rtype import RTLIRDataType as rdt
 from pymtl3.passes.rtlir.rtype import RTLIRType as rt
 
 from . import BehavioralRTLIR as bir
 from .BehavioralRTLIR import BaseBehavioralRTLIR
+from .BehavioralRTLIRGenL1Pass import BehavioralRTLIRGenL1Pass
 
 
-class BehavioralRTLIRTypeCheckL1Pass( BasePass ):
+class BehavioralRTLIRTypeCheckL1Pass( RTLIRPass ):
+
+  # Pass metadata
+
+  #: A dictionary that maps free variable names to (object, RTLIRType)
+  #:
+  #: Type: ``dict``; output
+  rtlir_freevars = MetadataKey()
+
+  #: A set of variable names that are accessed in the upblks
+  #:
+  #: Type: ``set(str)``; output
+  rtlir_accessed = MetadataKey()
+
   def __init__( s, translation_top ):
+    c = s.__class__
     s.tr_top = translation_top
-    if not hasattr( translation_top, "_rtlir_getter" ):
-      translation_top._rtlir_getter = rt.RTLIRGetter(cache=True)
+    if not translation_top.has_metadata( c.rtlir_getter ):
+      translation_top.set_metadata( c.rtlir_getter, rt.RTLIRGetter(cache=True) )
 
   def __call__( s, m ):
     """Perform type checking on all RTLIR in rtlir_upblks."""
-    if not hasattr( m, '_pass_behavioral_rtlir_type_check' ):
-      m._pass_behavioral_rtlir_type_check = PassMetadata()
-    m._pass_behavioral_rtlir_type_check.rtlir_freevars = {}
-    m._pass_behavioral_rtlir_type_check.rtlir_accessed = set()
-    visitor = s.get_visitor_class()(
+    c = s.__class__
+
+    rtlir_freevars = {}
+    rtlir_accessed = set()
+
+    m.set_metadata( BehavioralRTLIRTypeCheckL1Pass.rtlir_freevars, rtlir_freevars )
+    m.set_metadata( BehavioralRTLIRTypeCheckL1Pass.rtlir_accessed, rtlir_accessed )
+
+    type_checker = s.get_visitor_class()(
       m,
-      m._pass_behavioral_rtlir_type_check.rtlir_freevars,
-      m._pass_behavioral_rtlir_type_check.rtlir_accessed,
-      s.tr_top._rtlir_getter,
+      rtlir_freevars,
+      rtlir_accessed,
+      s.tr_top.get_metadata( c.rtlir_getter ),
     )
+
+    rtlir_upblks = m.get_metadata( BehavioralRTLIRGenL1Pass.rtlir_upblks )
+
     for blk in m.get_update_block_order():
-      visitor.enter( blk, m._pass_behavioral_rtlir_gen.rtlir_upblks[ blk ] )
+      type_checker.enter( blk, rtlir_upblks[ blk ] )
+
+  #-------------------------------------------------------------------------
+  # Type checker
+  #-------------------------------------------------------------------------
 
   def get_visitor_class( s ):
     return BehavioralRTLIRTypeCheckVisitorL1
@@ -50,6 +80,16 @@ class BehavioralRTLIRTypeCheckVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
     s.freevars = freevars
     s.accessed = accessed
     s.type_expect = {}
+
+    # The enforcer is called with a context and an AST node so that it can
+    # enforce every implicitly typed node in the subtree to have the type
+    # of the given context. Note that after one enforcement, the _is_explicit
+    # flag would be set to True.
+    try:
+      s.enforcer = s.get_enforce_visitor()( component )
+    except TypeError:
+      pass
+
     lhs_types = ( rt.Port, rt.Wire, rt.NetWire )
     index_types = ( rt.Port, rt.Wire, rt.Array )
     slice_types = ( rt.Port, rt.Wire )
@@ -101,14 +141,22 @@ class BehavioralRTLIRTypeCheckVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
         pass
     s.visit( rtlir )
 
+  def get_enforce_visitor( s ):
+    return BehavioralRTLIRTypeEnforcerL1
+
   # Override the default visit()
   def visit( s, node ):
+    # not node._is_explicit: is the parent node allowed to re-interpret this
+    # node's bitwidth without truncation?
     node_name = node.__class__.__name__
     method = 'visit_' + node_name
     func = getattr( s, method, s.generic_visit )
 
     # First visit (type check) all child nodes
     for field, value in vars(node).items():
+      # Special case For because we use context depedent types
+      # for the loop index
+      if node_name == 'For' and field == 'body': continue
       if isinstance( value, list ):
         for item in value:
           if isinstance( item, BaseBehavioralRTLIR ):
@@ -142,6 +190,10 @@ class BehavioralRTLIRTypeCheckVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
   # Override the default generic_visit()
   def generic_visit( s, node ):
     node.Type = None
+    # Is the parent node allowed to re-interpret the type of this node?
+    node._is_explicit = True
+    # Has this node re-interpreted the type of its child node?
+    node._has_reinterpreted = False
 
   def is_same( s, u, v ):
     """Return if the sub-AST at u and v are the same."""
@@ -150,18 +202,25 @@ class BehavioralRTLIRTypeCheckVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
     return u == v
 
   def _visit_Assign_single_target( s, node, target, i ):
-    rhs_type = node.value.Type.get_dtype()
     lhs_type = target.Type.get_dtype()
+    rhs_type = node.value.Type.get_dtype()
+
+    # At L1 it's always signal assignment
+    is_rhs_reinterpretable = not node.value._is_explicit
+    if is_rhs_reinterpretable and ((not lhs_type(rhs_type)) or (rhs_type != lhs_type)):
+      s.enforcer.enter( s.blk, target.Type, node.value )
+
+    rhs_type = node.value.Type.get_dtype()
 
     # Weak type checking (agreeable types)
     if not lhs_type( rhs_type ):
       raise PyMTLTypeError( s.blk, node.ast,
-        f'Unagreeable types between LHS and RHS (LHS target#{i} of {lhs_type} vs {rhs_type})!' )
+        f'Unagreeable types between LHS and RHS (LHS target#{i+1} of {lhs_type} vs {rhs_type})!' )
 
     # Strong type checking (same type)
     if rhs_type != lhs_type:
       raise PyMTLTypeError( s.blk, node.ast,
-        f'LHS and RHS of assignment should have the same type (LHS target#{i} of {lhs_type} vs {rhs_type})!' )
+        f'LHS and RHS of assignment should have the same type (LHS target#{i+1} of {lhs_type} vs {rhs_type})!' )
 
   def visit_Assign( s, node ):
     # RHS should have the same type as any of LHS
@@ -169,6 +228,7 @@ class BehavioralRTLIRTypeCheckVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
       s._visit_Assign_single_target( node, target, i )
 
     node.Type = None
+    node._is_explicit = True
 
   def visit_FreeVar( s, node ):
     try:
@@ -178,8 +238,9 @@ class BehavioralRTLIRTypeCheckVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
         f'{node.name} cannot be converted into a valid RTLIR object!' )
 
     if isinstance( t, rt.Const ) and isinstance( t.get_dtype(), rdt.Vector ):
-      node._value = mk_bits( t.get_dtype().get_length() )( node.obj )
+      node._value = int(node.obj)
     node.Type = t
+    node._is_explicit = False if isinstance(node.obj, int) else True
 
     if node.name not in s.freevars:
       s.freevars[ node.name ] = ( node.obj, t )
@@ -188,14 +249,17 @@ class BehavioralRTLIRTypeCheckVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
     # Mark this node as having type rt.Component
     # In L1 the `s` top component is the only possible base
     node.Type = s.rtlir_getter.get_rtlir( node.base )
+    node._is_explicit = True
     if not isinstance( node.Type, rt.Component ):
       raise PyMTLTypeError( s.blk, node.ast,
         f'{node} is not a rt.Component!' )
 
   def visit_Number( s, node ):
-    # By default, number literals have bitwidth of 32
+    # By default, number literals have the minimal bitwidth that can
+    # hold its value without truncation.
     node.Type = s.rtlir_getter.get_rtlir( node.value )
-    node._value = Bits32( node.value )
+    node._value = int(node.value)
+    node._is_explicit = False
 
   def visit_Concat( s, node ):
     nbits = 0
@@ -205,39 +269,46 @@ class BehavioralRTLIRTypeCheckVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
           f'{child} is not a signal!' )
       nbits += child.Type.get_dtype().get_length()
     node.Type = rt.NetWire( rdt.Vector( nbits ) )
+    node._is_explicit = True
 
   def visit_ZeroExt( s, node ):
-    try:
-      new_nbits = node.nbits._value
-    except AttributeError:
-      raise PyMTLTypeError( s.blk, node.ast,
-        f'{node.nbits} is not a constant number!' )
+    new_nbits = node.nbits
     child_type = node.value.Type
     old_nbits = child_type.get_dtype().get_length()
-    if new_nbits <= old_nbits:
+    if new_nbits < old_nbits:
       raise PyMTLTypeError( s.blk, node.ast,
-        f'{new_nbits} is not greater than {old_nbits}!' )
+        f'the target bitwidth {new_nbits} is less than the bitwidth of the operand ({old_nbits})!' )
     node.Type = copy.copy( child_type )
     node.Type.dtype = rdt.Vector( new_nbits )
+    node._is_explicit = True
 
   def visit_SignExt( s, node ):
-    try:
-      new_nbits = node.nbits._value
-    except AttributeError:
-      raise PyMTLTypeError( s.blk, node.ast,
-        f'{node.nbits} is not a constant number!' )
+    new_nbits = node.nbits
     child_type = node.value.Type
     old_nbits = child_type.get_dtype().get_length()
-    if new_nbits <= old_nbits:
+    if new_nbits < old_nbits:
       raise PyMTLTypeError( s.blk, node.ast,
-        f'{new_nbits} is not greater than {old_nbits}!' )
+        f'the target bitwidth {new_nbits} is less than the bitwidth of the operand ({old_nbits})!' )
     node.Type = copy.copy( child_type )
     node.Type.dtype = rdt.Vector( new_nbits )
+    node._is_explicit = True
+
+  def visit_Truncate( s, node ):
+    new_nbits = node.nbits
+    child_type = node.value.Type
+    old_nbits = child_type.get_dtype().get_length()
+    if new_nbits > old_nbits:
+      raise PyMTLTypeError( s.blk, node.ast,
+        f'the target bitwidth {new_nbits} is larger than the bitwidth of the operand ({old_nbits})!' )
+    node.Type = copy.copy( child_type )
+    node.Type.dtype = rdt.Vector( new_nbits )
+    node._is_explicit = True
 
   def visit_Reduce( s, node ):
     child_type = node.value.Type
     node.Type = copy.copy( child_type )
     node.Type.dtype = rdt.Vector( 1 )
+    node._is_explicit = True
 
   def visit_SizeCast( s, node ):
     nbits = node.nbits
@@ -247,6 +318,7 @@ class BehavioralRTLIRTypeCheckVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
     # be able to explicitly convert signals/constatns to different bitwidth.
     node.Type = copy.copy( Type )
     node.Type.dtype = rdt.Vector( nbits )
+    node._is_explicit = True
 
     try:
       node._value = node.value._value
@@ -266,32 +338,76 @@ class BehavioralRTLIRTypeCheckVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
         f'non-component attribute {node.attr} of type {node.value.Type} is not supported at L1!' )
     # value.attr has the type that is specified by the base
     node.Type = node.value.Type.get_property( node.attr )
+    if isinstance( node.Type, rt.Const ):
+      dtype = node.Type.get_dtype()
+      if isinstance( dtype, rdt.Vector ):
+        node._is_explicit = dtype.is_explicit()
+      else:
+        node._is_explicit = True
+    else:
+      node._is_explicit = True
+
+  def _handle_index_extension( s, node, value, idx, dscp, inclusive=True ):
+    expected_nbits = value.Type.get_index_width()
+    idx_nbits = idx.Type.get_dtype().get_length()
+    is_idx_reinterpretable = not idx._is_explicit
+
+    if not inclusive and hasattr( idx, '_value' ):
+      idx_nbits = s._get_nbits_from_value( idx._value-1 )
+
+    if idx_nbits > expected_nbits:
+      # Either bitwidth do not match or requires implicit truncation
+      raise PyMTLTypeError( s.blk, node.ast,
+        f'expects a {expected_nbits}-bit index but the given {dscp} has more ({idx_nbits}) bits!' )
+    elif idx_nbits < expected_nbits:
+      if is_idx_reinterpretable:
+        # Implicit zero-extension
+        s.enforcer.enter( s.blk, rt.NetWire(rdt.Vector(expected_nbits)), idx )
+      else:
+        # Bitwidth mismatch
+        raise PyMTLTypeError( s.blk, node.ast,
+          f'expects a {expected_nbits}-bit index but the given {dscp} has {idx_nbits} bits!' )
+    else:
+      if idx_nbits != idx.Type.get_dtype().get_length():
+        # If we used a different bitwidth then enforce it
+        s.enforcer.enter( s.blk, rt.NetWire(rdt.Vector(idx_nbits)), idx )
 
   def visit_Index( s, node ):
-    idx = None if not hasattr(node.idx, "_value") else node.idx._value
+    idx = None if not hasattr(node.idx, "_value") else int(node.idx._value)
     if isinstance( node.value.Type, rt.Array ):
       if idx is not None and not (0 <= idx < node.value.Type.get_dim_sizes()[0]):
         raise PyMTLTypeError( s.blk, node.ast, 'array index out of range!' )
       node.Type = node.value.Type.get_next_dim_type()
       obj = node.value.Type.get_obj()
+      s._handle_index_extension( node, node.value, node.idx, 'index' )
+
+      # If the given index yields an integer, mark this node as implicit
       if idx is not None and obj is not None:
         if isinstance( node.Type, rt.Array ):
           node.Type.obj = obj[ int( idx ) ]
+          node._is_explicit = True
         else:
-          node._value = obj[ int( idx ) ]
+          node._value = int( obj[ int( idx ) ] )
+          node._is_explicit = False if isinstance(node._value, int) else True
+      else:
+        node._is_explicit = True
 
     elif isinstance( node.value.Type, rt.Signal ):
       dtype = node.value.Type.get_dtype()
+      s._handle_index_extension( node, node.value, node.idx, 'index' )
+
       if node.value.Type.is_packed_indexable():
         if idx is not None and not (0 <= idx < dtype.get_length()):
           raise PyMTLTypeError( s.blk, node.ast,
             'bit selection index out of range!' )
         node.Type = node.value.Type.get_next_dim_type()
+        node._is_explicit = True
       elif isinstance( dtype, rdt.Vector ):
         if idx is not None and not(0 <= idx < dtype.get_length()):
           raise PyMTLTypeError( s.blk, node.ast,
             'bit selection index out of range!' )
         node.Type = rt.NetWire( rdt.Vector( 1 ) )
+        node._is_explicit = True
       else:
         raise PyMTLTypeError( s.blk, node.ast,
           f'cannot perform index on {dtype}!')
@@ -306,9 +422,16 @@ class BehavioralRTLIRTypeCheckVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
     upper_val = None if not hasattr(node.upper, "_value") else node.upper._value
     dtype = node.value.Type.get_dtype()
 
+    if hasattr(node.value, "_value"):
+      raise PyMTLTypeError( s.blk, node.ast,
+          f'cannot perform slicing on constant {node.value._value}!')
+
     if not isinstance( dtype, rdt.Vector ):
       raise PyMTLTypeError( s.blk, node.ast, f'cannot perform slicing on type {dtype}!')
 
+    s._handle_index_extension( node, node.value, node.lower, 'slice lower bound' )
+    s._handle_index_extension( node, node.value, node.upper, 'slice upper bound', False )
+
     if not lower_val is None and not upper_val is None:
       signal_nbits = dtype.get_length()
       # upper bound must be strictly larger than the lower bound
@@ -320,6 +443,7 @@ class BehavioralRTLIRTypeCheckVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
         raise PyMTLTypeError( s.blk, node.ast,
           'upper/lower bound of slice out of width of signal!' )
       node.Type = rt.NetWire( rdt.Vector( int( upper_val - lower_val ) ) )
+      node._is_explicit = True
 
     else:
       # Try to special case the constant-stride part selection
@@ -330,8 +454,66 @@ class BehavioralRTLIRTypeCheckVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
         slice_size = nbits._value
         assert s.is_same( node.lower, node.upper.left )
         node.Type = rt.NetWire( rdt.Vector( slice_size ) )
+        node._is_explicit = True
         # Add new fields that might help translation
         node.size = slice_size
         node.base = node.lower
       except Exception:
         raise PyMTLTypeError( s.blk, node.ast, 'slice bounds must be constant!' )
+
+  def _get_nbits_from_value( s, value ):
+    if -1 <= value <= 1:
+      return 1
+    if value < 0:
+      return math.ceil(math.log2(abs(value)))
+    else:
+      return math.ceil(math.log2(value+1))
+
+#-------------------------------------------------------------------------
+# Enforce types for all terms whose types are inferred (implicit)
+#-------------------------------------------------------------------------
+
+class BehavioralRTLIRTypeEnforcerL1( bir.BehavioralRTLIRNodeVisitor ):
+
+  def __init__( s, component ):
+    s.component = component
+
+  def enter( s, blk, context, node ):
+    s.blk = blk
+    s.stack = deque([])
+    with s.register_context( context ):
+      s.visit( node )
+
+  @contextmanager
+  def register_context( s, context_type ):
+    s.stack.append( context_type )
+    yield
+    s.stack.pop()
+
+  def get_context( s, node, obj ):
+    if not s.stack:
+      raise PyMTLTypeError( s.blk, node.ast,
+          f'no context was provided to validate the inferred bitwidth of {obj}!' )
+    return s.stack[-1]
+
+  def mutate_datatype( s, node, descp ):
+    if not node._is_explicit:
+      # assert isinstance(node.Type, rt.Const), f'internal error: {node} is not constant!'
+      target_Type = s.get_context(node, descp).get_dtype()
+      # All RTLIR datatypes are cached -- we don't want to invalidate the cache
+      # and therefore a deepcopy is needed here
+      node.Type = copy.deepcopy(node.Type)
+      node.Type.dtype = target_Type
+      # node._is_explicit = True
+
+  def visit_FreeVar( s, node ):
+    s.mutate_datatype( node, node.obj )
+
+  def visit_Number( s, node ):
+    s.mutate_datatype( node, node.value )
+
+  def visit_Attribute( s, node ):
+    s.mutate_datatype( node, node.attr )
+
+  def visit_Index( s, node ):
+    s.mutate_datatype( node, 'indexing' )
