@@ -41,7 +41,7 @@ def local_do_test( m ):
   try:
     ref = m._rtlir_test_ref
     for blk in m.get_update_blocks():
-      upblk = m._pass_behavioral_rtlir_gen.rtlir_upblks[ blk ]
+      upblk = m.get_metadata( BehavioralRTLIRGenL4Pass.rtlir_upblks )[ blk ]
       assert upblk == ref[ blk.__name__ ]
   except AttributeError:
     pass
