@@ -13,6 +13,7 @@ RTLIR data type module.
 """
 import copy
 import inspect
+import math
 
 import pymtl3.dsl as dsl
 from pymtl3.datatypes import Bits, is_bitstruct_inst
@@ -78,6 +79,14 @@ class Array( BaseRTLIRType ):
   def get_dim_sizes( s ):
     return s.dim_sizes
 
+  def get_index_width( s ):
+    assert s.dim_sizes, 'rt.Array is created without dimension!'
+    n_elements = s.dim_sizes[0]
+    if n_elements <= 1:
+      return 1
+    else:
+      return math.ceil(math.log2(n_elements))
+
   def get_sub_type( s ):
     return s.sub_type
 
@@ -111,6 +120,9 @@ class Signal( BaseRTLIRType ):
   def get_dtype( s ):
     return s.dtype
 
+  def get_index_width( s ):
+    return s.dtype.get_index_width()
+
   def _is_unpacked( s ):
     return s.unpacked
 
@@ -507,6 +519,7 @@ def is_rtlir_convertible( obj ):
 # a global cache that invalidates garbage collection
 
 NA = "<N/A>"
+
 # uncached=0
 class RTLIRGetter:
   ifc_primitive_types = ( dsl.InPort, dsl.OutPort, dsl.Interface )
